{
Copyright (c) 2010, Carl-Philip Hänsch

Permission to use, copy, modify, and/or distribute this software for any purpose with or
without fee is hereby granted, provided that the above copyright notice and this permission
notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
OR PERFORMANCE OF THIS SOFTWARE.
}


{
	* GLES-2.0 Tutorial
	* 
	* Hier zeige ich euch ein kleines Stück Code,
	* das einen Farbverlauf eines Dreiecks zeichnet.
	* 
	* 
	* 
	* }

program gles_tutorial;
{$mode objfpc}{$H+}
{$APPTYPE CONSOLE}

uses sdl, gles_header, sysutils, classes, gles_shaderapi;


const 
	vertexshader =  // Wir legen den Vertexshader an
				'attribute vec4 position;'+ // Der Shader bekommt Vertexdaten
				'attribute vec3 glcolor;'+ // Und pro Vertex einen Farbwert
				'varying vec3 color;'+ // Außerdem interpoliert er die Farbe in die Pixel
				'void main() {'+
				'gl_Position = position;'+ // Die Position wird 1:1 übernommen
				'color = glcolor;'+ // und der Interpolationswert festgelegt
				'}';
				
	fragmentshader = // und den Fragmentshader
				'varying vec3 color;'+ // Die interpolierte Farbe
				'void main() {'+
				'gl_FragColor = vec4(color, 1.0);'+ // Die Pixelfarbe wird auf den Farbwert gesetzt
				'}';
				
	points: array[0..2, 0..2] of single = ( // die 3 Vertices der Punkte
		(0, 1, 0), // Vertices
		(-1, 0, 0),
		(1, 0, 0));
		
	colors: array[0..2, 0..2] of single = ( // die 3 Farbwerte der Punkte
		(1, 0, 0), // Farben
		(0, 1, 0),
		(0, 0, 1));		
		
	width = 640; // Fenstergröße
	height = 480;
	
var vertex_loc, color_loc: cardinal; // Wo unsere Attribute sind
		
		
procedure Render;
begin
 glViewPort(0, 0, width, height); // Fensterzeichenbereich angeben
 glClearColor(0, 0, 0, 0); // Hintergrundfarbe schwarz
 glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT); // Farbe und Tiefenbuffer löschen
 
 glVertexAttribPointer(vertex_loc, 3, GL_FLOAT, 0, sizeof(single)*3, @points[0,0]); // Vertices eingeben
 glEnableVertexAttribArray(vertex_loc); // Vertices erlauben
 
 glVertexAttribPointer(color_loc, 3, GL_FLOAT, 0, sizeof(single)*3, @colors[0,0]); // Farbe eingeben
 glEnableVertexAttribArray(color_loc); // Farben erlauben
 
 glDrawArrays(GL_TRIANGLES, 0, 3); // Ein Dreieck zeichnen
end;		


var ev: TSDL_Event;	
	running: boolean = true;
	
	vertex_shader, fragment_shader, shaderprog: cardinal; // Die 2 Shader und das Shaderprogramm
begin
 writeln('Beginne Programm');
 SDL_Init(SDL_Init_Video); // SDL initialisieren
 writeln('SDL init: ', SDL_GetError);
 SDL_SetVideoMode(width, height, 0, SDL_OPENGL); // Bildschirmauflösung setzen und OpenGL-Context erstellen
 writeln('SDL videomode: ', SDL_GetError);
 writeln('GLES: ', InitOpenGL(false)); // Den GLES-Header laden
 
 vertex_shader:=glesutLoadShader(vertexshader, GL_VERTEX_SHADER); // Vertex-Shader kompilieren
 writeln(glesutGetInfoLog(vertex_shader)); // Fehlermeldungen ausgeben
 
 fragment_shader:=glesutLoadShader(fragmentshader, GL_FRAGMENT_SHADER); // Fragment-Shader kompilieren
 writeln(glesutGetInfoLog(fragment_shader)); // Fehlermeldungen ausgeben
 
 shaderprog:=glesutCreateProgram(vertex_shader, fragment_shader); // Beide Shader zusammenwürfeln
 writeln(glesutGetProgInfoLog(shaderprog)); // Fehlermeldungen ausgeben
 glDeleteShader(vertex_shader);
 glDeleteShader(fragment_shader);
 
 glUseProgram(shaderprog); // aktuellen Shader benutzen
 vertex_loc:=glGetAttribLocation(shaderprog, 'position'); // Vertex-position linken
 color_loc:=glGetAttribLocation(shaderprog, 'glcolor'); // Vertex-position linken
 
 while running do begin // Hauptschleife
  Render; // Render-Aufruf
  SDL_GL_Swapbuffers; // Die Buffer tauschen
 
  while SDL_PollEvent(@ev) > 0 do begin // Event-Verarbeitung
   case ev.type_ of 
    SDL_QUITEV: running:=false; // Einige Exit-Routinen
    SDL_KEYDOWN: begin
     case ev.key.keysym.sym of
      SDLK_ESCAPE: running:=false;
     
     
     end;    
    end;
   
   
   end;
  end;
  sleep(10); // ein kleines Delay, damit er sich nicht überhitzt
 end;
 
 glDeleteProgram(shaderprog);
 
 SDL_Quit; // SDL ordentlich beenden
end.
