{
Copyright (c) 2010, Carl-Philip Hänsch

Permission to use, copy, modify, and/or distribute this software for any purpose with or
without fee is hereby granted, provided that the above copyright notice and this permission
notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
OR PERFORMANCE OF THIS SOFTWARE.
}


{
	* GLES-2.0 Tutorial
	* 
	* Hier zeige ich euch ein kleines Stück Code,
	* das eine Textur auf ein Dreieck zeichnet.
	* 
	* 
	* 
	* }

program gles_tutorial;
{$mode objfpc}{$H+}
{$APPTYPE CONSOLE}

uses sdl, gles_header, sysutils, classes, gles_shaderapi;


const 
	vertexshader =  // Wir legen den Vertexshader an
				'attribute vec4 position;'+ // Der Shader bekommt Vertexdaten
				'attribute vec2 uv;'+ // Und pro Vertex einen UV-Wert
				'varying vec2 _uv;'+ // Außerdem interpoliert er die Tex-Koordinaten
				'void main() {'+
				'gl_Position = position;'+ // Die Position wird 1:1 übernommen
				'_uv = uv;'+ // und der Interpolationswert festgelegt
				'}';
				
	fragmentshader = // und den Fragmentshader
				'varying vec2 _uv;'+ // Die interpolierten Texturkoordinaten
				'uniform sampler2D tex;'+ // Die Textur
				'void main() {'+
				'gl_FragColor = texture2D(tex, _uv);'+ // Die Pixelfarbe aus der Textur gelesen
				'}';
				
	points: array[0..2, 0..2] of single = ( // die 3 Vertices der Punkte
		(0, 1, 0), // Vertices
		(-1, 0, 0),
		(1, 0, 0));
		
	uv: array[0..2, 0..1] of single = ( // die 3 UV-Koordinaten
		(1, 0), // UV
		(0, 1),
		(0, 0));		
		
	width = 640; // Fenstergröße
	height = 480;
	
var vertex_loc, uv_loc: cardinal; // Wo unsere Attribute sind
	tex: cardinal; //Die Textur
		
		
procedure Render;
begin
 glViewPort(0, 0, width, height); // Fensterzeichenbereich angeben
 glClearColor(0, 0, 1, 0); // Hintergrundfarbe schwarz
 glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT); // Farbe und Tiefenbuffer löschen
 
 glActiveTexture(GL_TEXTURE0); // mit Textur-Unit 0
 glBindTexture(GL_TEXTURE_2D, tex); // Binde Textur
 
 glVertexAttribPointer(vertex_loc, 3, GL_FLOAT, 0, sizeof(single)*3, @points[0,0]); // Vertices eingeben
 glEnableVertexAttribArray(vertex_loc); // Vertices erlauben
 
 glVertexAttribPointer(uv_loc, 2, GL_FLOAT, 0, sizeof(single)*2, @uv[0,0]); // UV eingeben
 glEnableVertexAttribArray(uv_loc); // Farben erlauben
 
 glDrawArrays(GL_TRIANGLES, 0, 3); // Ein Dreieck zeichnen
end;		


var ev: TSDL_Event;	
	running: boolean = true;
	
	vertex_shader, fragment_shader, shaderprog: cardinal; // Die 2 Shader und das Shaderprogramm
	
	mem: pointer;
	i: integer;
begin
 SDL_Init(SDL_Init_Video); // SDL initialisieren
 SDL_SetVideoMode(width, height, 0, SDL_OPENGL); // Bildschirmauflösung setzen und OpenGL-Context erstellen
 InitOpenGL(false); // Den GLES-Header laden
 
 vertex_shader:=glesutLoadShader(vertexshader, GL_VERTEX_SHADER); // Vertex-Shader kompilieren
 writeln(glesutGetInfoLog(vertex_shader)); // Fehlermeldungen ausgeben
 
 fragment_shader:=glesutLoadShader(fragmentshader, GL_FRAGMENT_SHADER); // Fragment-Shader kompilieren
 writeln(glesutGetInfoLog(fragment_shader)); // Fehlermeldungen ausgeben
 
 shaderprog:=glesutCreateProgram(vertex_shader, fragment_shader); // Beide Shader zusammenwürfeln
 writeln(glesutGetProgInfoLog(shaderprog)); // Fehlermeldungen ausgeben
 glDeleteShader(vertex_shader);
 glDeleteShader(fragment_shader);
 
 glUseProgram(shaderprog); // aktuellen Shader benutzen
 vertex_loc:=glGetAttribLocation(shaderprog, 'position'); // Vertex-position linken
 uv_loc:=glGetAttribLocation(shaderprog, 'uv'); // Vertex-position linken
 
 glUniform1i(glGetUniformLocation(shaderprog, 'tex'), 0); // Textur-Unit 0 zuweisen
 
 
 mem:=GetMem(256*256*4); // Texturdaten allokieren
 for i:=0 to 256*256-1 do (PCardinal(mem)+i)^:=$FF000000 or Random(1 shl 24); // Textur mit Zufall füllen
 
 glGenTextures(1, @tex); // Textur erstellen
 glBindTexture(GL_TEXTURE_2D, tex); // Binden
 glPixelStorei(GL_UNPACK_ALIGNMENT, 1); 
 glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); // Wichtig: Filter einstellen
 glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 256, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, mem); // Bilddaten an die GPU hochladen
 glGenerateMipmap(GL_TEXTURE_2D);
 FreeMem(mem); // RAM kann freigegeben werden
 
 while running do begin // Hauptschleife
  Render; // Render-Aufruf
  SDL_GL_Swapbuffers; // Die Buffer tauschen
 
  while SDL_PollEvent(@ev) > 0 do begin // Event-Verarbeitung
   case ev.type_ of 
    SDL_QUITEV: running:=false; // Einige Exit-Routinen
    SDL_KEYDOWN: begin
     case ev.key.keysym.sym of
      SDLK_ESCAPE: running:=false;
     
     
     end;    
    end;
   
   
   end;
  end;
  sleep(10); // ein kleines Delay, damit er sich nicht überhitzt
 end;
 
 glDeleteTextures(1, @tex);
 glDeleteProgram(shaderprog);
 
 SDL_Quit; // SDL ordentlich beenden
end.
